@@ -1,7 +1,6 @@
 """CLI function for a vehicle."""
 
 from typing import Any
-from typing import Dict
 from typing import Optional
 
 import aiohttp
@@ -16,7 +15,7 @@ from renault_api.cli import renault_vehicle
 @click.pass_obj
 @helpers.coro_with_websession
 async def history(
-    ctx_data: Dict[str, Any],
+    ctx_data: dict[str, Any],
     *,
     start: str,
     end: str,
@@ -41,7 +40,7 @@ async def history(
 @click.pass_obj
 @helpers.coro_with_websession
 async def sessions(
-    ctx_data: Dict[str, Any],
+    ctx_data: dict[str, Any],
     *,
     start: str,
     end: str,
