@@ -60,3 +60,9 @@ class FailedForwardException(KamereonResponseException):
     """Failed to forward request to remote service."""
 
     pass
+
+
+class ForbiddenException(KamereonResponseException):
+    """The access is forbidden."""
+
+    pass
