@@ -1,10 +1,10 @@
 """Kamereon models."""
 
 import json
+import logging
+from collections.abc import Mapping
 from dataclasses import dataclass
 from typing import Any
-from typing import Dict
-from typing import List
 from typing import Optional
 from typing import cast
 
@@ -16,7 +16,9 @@ from . import helpers
 from .enums import AssetPictureSize
 from renault_api.models import BaseModel
 
-COMMON_ERRRORS: List[Dict[str, Any]] = [
+_LOGGER = logging.getLogger(__name__)
+
+COMMON_ERRRORS: list[dict[str, Any]] = [
     {
         "errorCode": "err.func.400",
         "error_type": exceptions.InvalidInputException,
@@ -45,44 +47,137 @@ COMMON_ERRRORS: List[Dict[str, Any]] = [
         "errorCode": "err.func.wired.overloaded",
         "error_type": exceptions.QuotaLimitException,
     },
+    {
+        "errorCode": "err.func.wired.forbidden",
+        "error_type": exceptions.ForbiddenException,
+    },
 ]
 
-VEHICLE_SPECIFICATIONS: Dict[str, Dict[str, Any]] = {
+VEHICLE_SPECIFICATIONS: dict[str, dict[str, Any]] = {
     "X101VE": {  # ZOE phase 1
         "reports-charge-session-durations-in-minutes": True,
         "reports-in-watts": True,
-        "support-endpoint-location": False,
-        "support-endpoint-lock-status": False,
-    },
-    "X102VE": {  # ZOE phase 2
-        "warns-on-method-set_ac_stop": "Action `cancel` on endpoint `hvac-start` may not be supported on this model.",  # noqa
-    },
-    "XJA1VP": {  # CLIO V
-        "support-endpoint-hvac-status": False,
-    },
-    "XJB1SU": {  # CAPTUR II
-        "support-endpoint-hvac-status": False,
     },
     "XBG1VE": {  # DACIA SPRING
         "control-charge-via-kcm": True,
     },
-    "XCB1VE": {  # MEGANE E-TECH
-        "support-endpoint-lock-status": False,
-    },
-    "XCB1SE": {  # SCENIC E-TECH
-        "support-endpoint-lock-status": False,
-    },
 }
 
-GATEWAY_SPECIFICATIONS: Dict[str, Dict[str, Any]] = {
+GATEWAY_SPECIFICATIONS: dict[str, dict[str, Any]] = {
     "GDC": {  # ZOE phase 1
         "reports-charge-session-durations-in-minutes": True,
         "reports-in-watts": True,
-        "support-endpoint-location": False,
-        "support-endpoint-lock-status": False,
     },
 }
 
+_DEFAULT_ENDPOINTS: dict[str, str] = {
+    "alerts": "/vehicles/{vin}/alerts",
+    "battery-status": "/kca/car-adapter/v2/cars/{vin}/battery-status",
+    "charge-history": "/kca/car-adapter/v1/cars/{vin}/charge-history",
+    "charge-mode": "/kca/car-adapter/v1/cars/{vin}/charge-mode",
+    "charge-schedule": "/kca/car-adapter/v1/cars/{vin}/charge-schedule",
+    "charges": "/kca/car-adapter/v1/cars/{vin}/charges",
+    "charging-settings": "/kca/car-adapter/v1/cars/{vin}/charging-settings",
+    "cockpit": "/kca/car-adapter/v1/cars/{vin}/cockpit",
+    "hvac-history": "/kca/car-adapter/v1/cars/{vin}/hvac-history",
+    "hvac-sessions": "/kca/car-adapter/v1/cars/{vin}/hvac-sessions",
+    "hvac-settings": "/kca/car-adapter/v1/cars/{vin}/hvac-settings",
+    "hvac-status": "/kca/car-adapter/v1/cars/{vin}/hvac-status",
+    "location": "/kca/car-adapter/v1/cars/{vin}/location",
+    "lock-status": "/kca/car-adapter/v1/cars/{vin}/lock-status",
+    "notification-settings": "/kca/car-adapter/v1/cars/{vin}/notification-settings",
+    "pressure": "/kca/car-adapter/v1/cars/{vin}/pressure",
+    "res-state": "/kca/car-adapter/v1/cars/{vin}/res-state",
+    "soc-levels": "/kcm/v1/vehicles/{vin}/ev/soc-levels",
+}
+_KCM_ENDPOINTS: dict[str, str] = {
+    "charge-schedule": "/kcm/v1/vehicles/{vin}/ev/settings"
+}
+
+_VEHICLE_ENDPOINTS: dict[str, dict[str, Optional[str]]] = {
+    "R5E1VE": {  # Renault 5 E-TECH
+        "charge-schedule": _KCM_ENDPOINTS["charge-schedule"],
+    },
+    "X071VE": {  # TWINGO III
+        "res-state": None,  # not supported
+        "lock-status": None,  # seems not supported (404)
+    },
+    "X101VE": {  # ZOE phase 1
+        "battery-status": _DEFAULT_ENDPOINTS["battery-status"],  # confirmed
+        "charge-mode": _DEFAULT_ENDPOINTS["charge-mode"],  # confirmed
+        "charge-schedule": _DEFAULT_ENDPOINTS["charge-schedule"],  # confirmed
+        "cockpit": _DEFAULT_ENDPOINTS["cockpit"],  # confirmed
+        "hvac-status": _DEFAULT_ENDPOINTS["hvac-status"],  # confirmed
+        "location": None,  # not supported
+        "lock-status": None,  # not supported
+        "pressure": None,  # not supported
+        "res-state": None,  # not supported
+        "soc-levels": None,  # not supported
+    },
+    "X102VE": {  # ZOE phase 2
+        "res-state": None,  # not supported
+        "lock-status": None,  # seems not supported (404)
+    },
+    "XBG1VE": {  # DACIA SPRING
+        "lock-status": None,
+        "res-state": None,
+        "charge-mode": None,
+    },
+    "XCB1SE": {  # SCENIC E-TECH
+        "charge-schedule": _KCM_ENDPOINTS["charge-schedule"],
+        "lock-status": None,
+    },
+    "XCB1VE": {  # MEGANE E-TECH
+        "lock-status": None,
+    },
+    "XJA1VP": {  # CLIO V
+        "hvac-status": None,
+    },
+    "XJB1SU": {  # CAPTUR II
+        "hvac-status": None,
+    },
+}
+
+_ALREADY_WARNED_VEHICLE: set[str] = set()
+_ALREADY_WARNED_VEHICLE_ENDPOINT: set[str] = set()
+
+
+def get_model_endpoints(model_code: Optional[str]) -> Mapping[str, Optional[str]]:
+    """Return model endpoints."""
+    if not model_code:
+        # Model code not available
+        return _DEFAULT_ENDPOINTS
+
+    if model_code not in _VEHICLE_ENDPOINTS:
+        # Model not documented
+        if model_code not in _ALREADY_WARNED_VEHICLE:
+            _ALREADY_WARNED_VEHICLE.add(model_code)
+            _LOGGER.warning(
+                "Model %s is not documented, using default endpoints", model_code
+            )
+        return _DEFAULT_ENDPOINTS
+
+    return _VEHICLE_ENDPOINTS[model_code]
+
+
+def get_model_endpoint(model_code: Optional[str], endpoint: str) -> Optional[str]:
+    """Return model endpoint"""
+    endpoints = get_model_endpoints(model_code)
+
+    if endpoint not in endpoints:
+        # Endpoint not documented
+        key = f"{model_code}:{endpoint}"
+        if key not in _ALREADY_WARNED_VEHICLE_ENDPOINT:
+            _ALREADY_WARNED_VEHICLE_ENDPOINT.add(key)
+            _LOGGER.warning(
+                "Endpoint %s for model %s is not documented, using default endpoints",
+                endpoint,
+                model_code,
+            )
+        return _DEFAULT_ENDPOINTS.get(endpoint)
+
+    return endpoints[endpoint]
+
 
 @dataclass
 class KamereonResponseError(BaseModel):
@@ -98,9 +193,7 @@ class KamereonResponseError(BaseModel):
             if self.errorCode == common_error["errorCode"]:
                 error_type = common_error["error_type"]
                 raise error_type(self.errorCode, error_details)
-        raise exceptions.KamereonResponseException(
-            self.errorCode, error_details
-        )  # pragma: no cover
+        raise exceptions.KamereonResponseException(self.errorCode, error_details)
 
     def get_error_details(self) -> Optional[str]:
         """Extract the error details sometimes hidden inside nested JSON."""
@@ -130,7 +223,7 @@ class KamereonResponseError(BaseModel):
 class KamereonResponse(BaseModel):
     """Kamereon response."""
 
-    errors: Optional[List[KamereonResponseError]]
+    errors: Optional[list[KamereonResponseError]]
 
     def raise_for_error_code(self) -> None:
         """Raise exception if errors found in the response."""
@@ -152,7 +245,7 @@ class KamereonPersonAccount(BaseModel):
 class KamereonPersonResponse(KamereonResponse):
     """Kamereon response to GET on /persons/{gigya_person_id}."""
 
-    accounts: Optional[List[KamereonPersonAccount]]
+    accounts: Optional[list[KamereonPersonAccount]]
 
 
 @dataclass
@@ -175,7 +268,7 @@ class KamereonVehicleDetails(BaseModel):
     model: Optional[KamereonVehicleDetailsGroup]
     energy: Optional[KamereonVehicleDetailsGroup]
     engineEnergyType: Optional[str]
-    assets: Optional[List[Dict[str, Any]]]
+    assets: Optional[list[dict[str, Any]]]
 
     def get_energy_code(self) -> Optional[str]:
         """Return vehicle energy code."""
@@ -193,7 +286,7 @@ class KamereonVehicleDetails(BaseModel):
         """Return vehicle model label."""
         return self.model.label if self.model else None
 
-    def get_asset(self, asset_type: str) -> Optional[Dict[str, Any]]:
+    def get_asset(self, asset_type: str) -> Optional[dict[str, Any]]:
         """Return asset."""
         return next(
             filter(
@@ -205,9 +298,9 @@ class KamereonVehicleDetails(BaseModel):
         self, size: AssetPictureSize = AssetPictureSize.LARGE
     ) -> Optional[str]:
         """Return vehicle picture."""
-        asset: Dict[str, Any] = self.get_asset("PICTURE") or {}
+        asset: dict[str, Any] = self.get_asset("PICTURE") or {}
 
-        rendition: Dict[str, str] = next(
+        rendition: dict[str, str] = next(
             filter(
                 lambda rendition: rendition.get("resolutionType")
                 == f"ONE_MYRENAULT_{size.name}",
@@ -246,7 +339,7 @@ class KamereonVehicleDetails(BaseModel):
             return VEHICLE_SPECIFICATIONS.get(  # type:ignore[no-any-return]
                 self.model.code, {}
             ).get("reports-charge-session-durations-in-minutes", False)
-        return False  # pragma: no cover
+        return False
 
     def reports_charging_power_in_watts(self) -> bool:
         """Return True if model reports chargingInstantaneousPower in watts."""
@@ -255,25 +348,12 @@ class KamereonVehicleDetails(BaseModel):
             return VEHICLE_SPECIFICATIONS.get(  # type:ignore[no-any-return]
                 self.model.code, {}
             ).get("reports-in-watts", False)
-        return False  # pragma: no cover
+        return False
 
     def supports_endpoint(self, endpoint: str) -> bool:
         """Return True if model supports specified endpoint."""
         # Default to True for unknown vehicles
-        if self.model and self.model.code:
-            return VEHICLE_SPECIFICATIONS.get(  # type:ignore[no-any-return]
-                self.model.code, {}
-            ).get(f"support-endpoint-{endpoint}", True)
-        return True  # pragma: no cover
-
-    def warns_on_method(self, method: str) -> Optional[str]:
-        """Return warning message if model trigger a warning on the method call."""
-        # Default to None for unknown vehicles
-        if self.model and self.model.code:
-            return VEHICLE_SPECIFICATIONS.get(  # type:ignore[no-any-return]
-                self.model.code, {}
-            ).get(f"warns-on-method-{method}", None)
-        return None  # pragma: no cover
+        return self.get_endpoint(endpoint) is not None
 
     def controls_action_via_kcm(self, action: str) -> bool:
         """Return True if model uses endpoint via kcm."""
@@ -282,7 +362,15 @@ class KamereonVehicleDetails(BaseModel):
             return VEHICLE_SPECIFICATIONS.get(  # type:ignore[no-any-return]
                 self.model.code, {}
             ).get(f"control-{action}-via-kcm", False)
-        return False  # pragma: no cover
+        return False
+
+    def get_endpoints(self) -> Mapping[str, Optional[str]]:
+        """Return model endpoints."""
+        return get_model_endpoints(self.get_model_code())
+
+    def get_endpoint(self, endpoint: str) -> Optional[str]:
+        """Return model endpoint"""
+        return get_model_endpoint(self.get_model_code(), endpoint)
 
 
 @dataclass
@@ -299,7 +387,7 @@ class KamereonVehiclesResponse(KamereonResponse):
 
     accountId: Optional[str]
     country: Optional[str]
-    vehicleLinks: Optional[List[KamereonVehiclesLink]]
+    vehicleLinks: Optional[list[KamereonVehiclesLink]]
 
 
 @dataclass
@@ -332,7 +420,7 @@ class KamereonVehicleContract(BaseModel):
 class KamereonVehicleContractsResponse(KamereonResponse):
     """Kamereon response to GET on /accounts/{accountId}/vehicles/{vin}/contracts."""
 
-    contractList: Optional[List[KamereonVehicleContract]]
+    contractList: Optional[list[KamereonVehicleContract]]
 
 
 @dataclass
@@ -341,7 +429,7 @@ class KamereonVehicleData(BaseModel):
 
     type: Optional[str]
     id: Optional[str]
-    attributes: Optional[Dict[str, Any]]
+    attributes: Optional[dict[str, Any]]
 
 
 @dataclass
@@ -381,11 +469,9 @@ class KamereonVehicleBatteryStatusData(KamereonVehicleDataAttributes):
                 if self.plugStatus is not None
                 else None
             )
-        except ValueError as err:  # pragma: no cover
-            # should we return PlugState.NOT_AVAILABLE?
-            raise exceptions.KamereonException(
-                f"Unable to convert `{self.plugStatus}` to PlugState."
-            ) from err
+        except ValueError:
+            _LOGGER.warning("Unable to convert `%s` to PlugState.", self.plugStatus)
+            return None
 
     def get_charging_status(self) -> Optional[enums.ChargeState]:
         """Return charging status."""
@@ -395,11 +481,11 @@ class KamereonVehicleBatteryStatusData(KamereonVehicleDataAttributes):
                 if self.chargingStatus is not None
                 else None
             )
-        except ValueError as err:  # pragma: no cover
-            # should we return ChargeState.NOT_AVAILABLE?
-            raise exceptions.KamereonException(
-                f"Unable to convert `{self.chargingStatus}` to ChargeState."
-            ) from err
+        except ValueError:
+            _LOGGER.warning(
+                "Unable to convert `%s` to ChargeState.", self.chargingStatus
+            )
+            return None
 
 
 @dataclass
@@ -527,16 +613,7 @@ class KamereonVehicleCarAdapterData(KamereonVehicleDataAttributes):
             return GATEWAY_SPECIFICATIONS.get(  # type:ignore[no-any-return]
                 self.carGateway, {}
             ).get("reports-in-watts", False)
-        return False  # pragma: no cover
-
-    def supports_endpoint(self, endpoint: str) -> bool:
-        """Return True if model supports specified endpoint."""
-        # Default to True for unknown vehicles
-        if self.carGateway:
-            return GATEWAY_SPECIFICATIONS.get(  # type:ignore[no-any-return]
-                self.carGateway, {}
-            ).get(f"support-endpoint-{endpoint}", True)
-        return True  # pragma: no cover
+        return False
 
     def controls_action_via_kcm(self, action: str) -> bool:
         """Return True if model uses endpoint via kcm."""
@@ -545,7 +622,7 @@ class KamereonVehicleCarAdapterData(KamereonVehicleDataAttributes):
             return VEHICLE_SPECIFICATIONS.get(  # type:ignore[no-any-return]
                 self.modelCodeDetail, {}
             ).get(f"control-{action}-via-kcm", False)
-        return False  # pragma: no cover
+        return False
 
 
 @dataclass
@@ -555,7 +632,7 @@ class ChargeDaySchedule(BaseModel):
     startTime: Optional[str]
     duration: Optional[int]
 
-    def for_json(self) -> Dict[str, Any]:
+    def for_json(self) -> dict[str, Any]:
         """Create dict for json."""
         return {
             "startTime": self.startTime,
@@ -564,7 +641,7 @@ class ChargeDaySchedule(BaseModel):
 
     def get_end_time(self) -> Optional[str]:
         """Get end time."""
-        if self.startTime is None:  # pragma: no cover
+        if self.startTime is None:
             return None
         return helpers.get_end_time(self.startTime, self.duration)
 
@@ -583,9 +660,9 @@ class ChargeSchedule(BaseModel):
     saturday: Optional[ChargeDaySchedule]
     sunday: Optional[ChargeDaySchedule]
 
-    def for_json(self) -> Dict[str, Any]:
+    def for_json(self) -> dict[str, Any]:
         """Create dict for json."""
-        result: Dict[str, Any] = {
+        result: dict[str, Any] = {
             "id": self.id,
             "activated": self.activated,
         }
@@ -604,7 +681,7 @@ class HvacDaySchedule(BaseModel):
 
     readyAtTime: Optional[str]
 
-    def for_json(self) -> Dict[str, Optional[str]]:
+    def for_json(self) -> dict[str, Optional[str]]:
         """Create dict for json."""
         return {
             "readyAtTime": self.readyAtTime,
@@ -625,9 +702,9 @@ class HvacSchedule(BaseModel):
     saturday: Optional[HvacDaySchedule]
     sunday: Optional[HvacDaySchedule]
 
-    def for_json(self) -> Dict[str, Any]:
+    def for_json(self) -> dict[str, Any]:
         """Create dict for json."""
-        result: Dict[str, Any] = {
+        result: dict[str, Any] = {
             "id": self.id,
             "activated": self.activated,
         }
@@ -645,19 +722,19 @@ class KamereonVehicleChargingSettingsData(KamereonVehicleDataAttributes):
     """Kamereon vehicle data charging-settings attributes."""
 
     mode: Optional[str]
-    schedules: Optional[List[ChargeSchedule]]
+    schedules: Optional[list[ChargeSchedule]]
 
-    def update(self, args: Dict[str, Any]) -> None:
+    def update(self, args: dict[str, Any]) -> None:
         """Update schedule."""
-        if "id" not in args:  # pragma: no cover
+        if "id" not in args:
             raise ValueError("id not provided for update.")
-        if self.schedules is None:  # pragma: no cover
+        if self.schedules is None:
             self.schedules = []
         for schedule in self.schedules:
-            if schedule.id == args["id"]:  # pragma: no branch
+            if schedule.id == args["id"]:
                 helpers.update_charge_schedule(schedule, args)
                 return
-        self.schedules.append(helpers.create_charge_schedule(args))  # pragma: no cover
+        self.schedules.append(helpers.create_charge_schedule(args))
 
 
 @dataclass
@@ -665,19 +742,19 @@ class KamereonVehicleHvacSettingsData(KamereonVehicleDataAttributes):
     """Kamereon vehicle data hvac-settings (mode+schedules) attributes."""
 
     mode: Optional[str]
-    schedules: Optional[List[HvacSchedule]]
+    schedules: Optional[list[HvacSchedule]]
 
-    def update(self, args: Dict[str, Any]) -> None:
+    def update(self, args: dict[str, Any]) -> None:
         """Update schedule."""
-        if "id" not in args:  # pragma: no cover
+        if "id" not in args:
             raise ValueError("id not provided for update.")
-        if self.schedules is None:  # pragma: no cover
+        if self.schedules is None:
             self.schedules = []
         for schedule in self.schedules:
-            if schedule.id == args["id"]:  # pragma: no branch
+            if schedule.id == args["id"]:
                 helpers.update_hvac_schedule(schedule, args)
                 return
-        self.schedules.append(helpers.create_hvac_schedule(args))  # pragma: no cover
+        self.schedules.append(helpers.create_hvac_schedule(args))
 
 
 @dataclass
